class Singleton(type):
    def __init__(self, *args, **kw) -> None: ...
    def __call__(self, *args, **kw): ...

class Cached(type):
    def __init__(self, *args, **kw) -> None: ...
    def __call__(self, *args): ...
