from _typeshed import Incomplete
from typing_extensions import Final

class ReadOnlyCell:
    parent: Incomplete
    row: Incomplete
    column: Incomplete
    data_type: Incomplete
    def __init__(self, sheet, row, column, value, data_type: str = "n", style_id: int = 0) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    # defined twice in the implementation
    @property
    def coordinate(self): ...
    @property
    def column_letter(self): ...
    @property
    def style_array(self): ...
    @property
    def has_style(self): ...
    @property
    def number_format(self): ...
    @property
    def font(self): ...
    @property
    def fill(self): ...
    @property
    def border(self): ...
    @property
    def alignment(self): ...
    @property
    def protection(self): ...
    @property
    def is_date(self): ...
    @property
    def internal_value(self): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value) -> None: ...

class EmptyCell:
    value: Incomplete
    is_date: bool
    font: Incomplete
    border: Incomplete
    fill: Incomplete
    number_format: Incomplete
    alignment: Incomplete
    data_type: str

EMPTY_CELL: Final[EmptyCell]
