from _typeshed import Incomplete

from openpyxl.descriptors.serialisable import Serialisable

class ChartRelation(Serialisable):
    tagname: str
    namespace: Incomplete
    id: Incomplete
    def __init__(self, id) -> None: ...
