from _typeshed import Incomplete
from collections.abc import Generator

def dataframe_to_rows(df, index: bool = True, header: bool = True) -> Generator[Incomplete, None, None]: ...
def expand_index(index, header: bool = False) -> Generator[Incomplete, None, None]: ...
